#ifndef HARKIO_HARKCONFIG_SOURCE_HPP_
#define HARKIO_HARKCONFIG_SOURCE_HPP_

#include <harkio2/Vector3.hpp>
#include <harkio2/defineitem.hpp>
#include <harkio2/Position.hpp>
#include <harkio2/harkconfig/File.hpp>
#include <vector>
#include <glibmm/ustring.h>

/** @file Source.hpp
 * @brief Source class and subclasses definitions.
 */

namespace harkio2
{
namespace harkconfig
{

/** @brief A sound source information.
 * This class specifies information about a sound source.
 */
class Source
{
public:
    virtual ~Source() {}

    /** @brief Returns the data structure version.
     * Returns the version which this instance actually represents.
     * @return The version of this structure.
     */
    virtual unsigned int GetVersion() const = 0;
};

/** @brief A sound source information version 1.
 */
class Source1 : public Source
{
public:
    /** @brief Returns the source version.
     * @return The version which this class represents.
     */
    virtual unsigned int GetVersion() const { return 1; }

    /// @brief The position of this sound source.
    DEFINE_ITEM(pos, Vector3<float>, Pos, const Vector3<float>&)

    /// @brief The sound file name of this sound source.
    DEFINE_ITEM(file, std::string, File, const std::string&)

    /// @brief The start position in the sound file.
    DEFINE_ITEM(cutStart, unsigned int, CutStart, unsigned int)

    /// @brief The end position in the sound file.
    DEFINE_ITEM(cutEnd, unsigned int, CutEnd, unsigned int)

};

/** A sound source information version 2.
 */
class Source2 : public Source
{
public:
    /** @brief Returns the source version.
     * @return The version which this class represents.
     */
    virtual unsigned int GetVersion() const { return 2; }

    /// @brief The position or direction of this sound source.
    DEFINE_ITEM(position, Position<float>, Position, const Position<float>&)

    /// @brief The sound file names of this sound source.
    DEFINE_ITEM(files, std::vector<File1>, Files, const std::vector<File1>&)

    /// @brief The start position in the sound file.
    DEFINE_ITEM(cutStart, unsigned int, CutStart, unsigned int)

    /// @brief The end position in the sound file.
    DEFINE_ITEM(cutEnd, unsigned int, CutEnd, unsigned int)

    /// @brief A comment.
    DEFINE_ITEM(comment, Glib::ustring, Comment, const Glib::ustring&)

};

}
}

#endif // HARKIO_HARKCONFIG_SOURCE_HPP_

